.include "8515def.inc"
; used 8515 code system sony  for 2 x 16 display
; + keyboard
; + PWM modulace jasu displeje & EEPROM save

.equ stacktop       = 0x8f    ; !
.equ pulse_0_min    = 28 ; 675u         Header=4T   2000u
.equ pulse_0_max    = 42 ; 825u         1     =3T   1000u
.equ pulse_1_min    = 53 ; 1350u        0     =1.5T 750u
.equ pulse_1_max    = 69 ; 1650u        T     =     500u
.equ pulse_h_min    = 80 ; 1800u        Timer =     32u
.equ pulse_h_max    = 95 ; 3300u
.equ lcd_b_en       = 4  ; bity PB
.equ lcd_b_rw       = 5  ;
.equ lcd_b_rs       = 7  ; !
.equ lcd_b_light    = 6  ; !
.equ ir_val_rec     = 0  ; ir record valid
.equ f_rxproc       = 1  ; rx in progress
.equ disp_up        = 2  ; lcd transfer in progress
.equ ir_tran        = 3  ; ir transfer in progress
.equ header         = 4  ; ir header
.equ f_uart_rx      = 5  ; rx done
.equ val_rec        = 6  ; ir bit$ receive
.equ val_kbd        = 7  ; kbdcode OK
.equ linesize       = 16
.equ c_baud         = 25 ; 51 9600 Bd 34 14400 12 38400  16  28800  25  19200
.equ row1           = 3  ; keypad input rows
.equ row2           = 2
.equ row3           = 1  ; port A pins
.equ row4           = 0
.equ col1           = 7  ; keypad output columns
.equ col2           = 6
.equ col3           = 5
.equ col4           = 4
.equ steps          = 12   ; No of steps MIN-MAX b/l bright..
.equ diff           = (256/steps)   ; step val for OCR regs.
.equ bri_default    = 128  ; default brightness 50%
.equ step_pl        = 0x09 ; real kbd command code$
.equ skip_pl        = 0x0a
.equ step_mi        = 0x0d
.equ skip_mi        = 0x0e
.equ enter          = 0x0b

.def pmreg          = r0
.def inttemp        = r1
.def stepreg        = r2  ;  PWM step reg.
.def drem16ul       = r3
.def drem16uh       = r4
.def dres16ul       = r5
.def dres16uh       = r6
.def dd16ul         = r5
.def dd16uh         = r6
.def dv16ul         = r7
.def dv16uh         = r8
.def delayh         = r9
.def delayl         = r10
.def r_irbitpos     = r11
.def null           = r12
.def eins           = r13
.def ff             = r14
.def r_ircodecount  = r15
.def itemp1         = r16
.def itemp2         = r17
.def flags          = r18
.def dcnt16u        = r19
.def temp1          = r20
.def temp2          = r21
.def temp3          = r22
.def temp4          = r23
.def lcddata        = r24
.def lcdcmd         = r25
.def xaddrl         = r26
.def xaddrh         = r27
.def yaddrl         = r28
.def yaddrh         = r29
.def zaddrl         = r30
.def zaddrh         = r31

     .dseg
     .org stacktop+1

v_controldata:      .byte     1           ; send by master
v_rxbuff:           .byte     linesize*2+6; send by master
v_displaydata:      .byte     linesize*2+6; send to display
v_uart_rxptr:       .byte     2           ; recieve ptr
v_uart_rxsize:      .byte     1           ; recieved bytes
v_uart_txfifo:      .byte     8           ; transmit fifo
v_ircode:           .byte     1           ; real IR code
v_irresult:         .byte     3           ; 1st IR type detect 2nd command
v_keycode:          .byte     1           ; 1st key press
v_keycode2:         .byte     1           ; verify key
real_kbd:           .byte     1           ; real KBD code

      .cseg
          rjmp      reset
          rjmp      int_0     ; ext int 0
          reti                ; ext int 1
          reti                ; timer 1 capt. event
          reti                ; timer 1 comp. A event
          reti                ; timer 1 comp. B event
          reti                ; timer 1 overflow
          reti                ; timer 0 overflow
          reti                ; SPI transfer int
          rjmp      uart_rx   ; Uart Rx complete
          reti                ; Uart UDR empty
          reti                ; Uart Tx complete
          reti                ; Analog comp. handler

uart_rx:  in        inttemp,sreg
          in        itemp1,udr
          sbrc      flags,f_uart_rx
          rjmp      rx_e             ; ignore data until processed
          sbr       flags,1<<f_rxproc; disable irc process
          push      yaddrl
          push      yaddrh
          lds       yaddrl,v_uart_rxptr
          lds       yaddrh,v_uart_rxptr+1
          lds       itemp2,v_uart_rxsize
          tst       itemp2
          brne      rx_nf
          sbis      ucr,rxb8         ; start byte
          rjmp      rx_e1
          ldi       yaddrl,v_controldata
          ldi       yaddrh,v_controldata>>8
rx_nf:    inc       itemp2
          cpi       itemp2,linesize*2+1
          brlo      rx_inp
          sbr       flags,1<<f_uart_rx      ; ! EOT
          clr       itemp2
rx_inp:   sts       v_uart_rxsize,itemp2
          st        y+,itemp1
          sts       v_uart_rxptr,yaddrl
          sts       v_uart_rxptr+1,yaddrh
rx_e1:    cbr       flags,1<<f_rxproc
          pop       yaddrh
          pop       yaddrl
rx_e:     out       sreg,inttemp
          reti

int0dir:  out       gimsk,null
          ldi       itemp2,0x01
          rol       itemp2
          out       mcucr,itemp2
          ldi       itemp2,0x40
          out       gimsk,itemp2
          ret

uart_addtx:sbis     usr,udre           ; wait for Tx byte complete
          rjmp      uart_addtx         ;
          out       udr,temp1          ; send byte
          ret

int_0:    in        inttemp,sreg
          in        itemp1,tcnt0
          out       tcnt0,null         ; reset timer 0
          sbrc      flags,disp_up      ; ! wait for display !
          rjmp      norec
          sbrc      flags,f_rxproc     ; ! wait for rx done
          rjmp      norec              ; erase command !!
          sbrc      flags,val_rec      ; 1=OK
          rjmp      ir_rec
          mov       r_irbitpos,null    ; Reset bitcounter
          clc                          ; Set falling edge
          rcall     int0dir
          sbr       flags,1<<header
          sbr       flags,1<<val_rec
          rjmp      int0_e

ir_rec:   sbrs      flags,header
          rjmp      ir_recdata
          cpi       itemp1,pulse_h_min
          brlo      norec
          cpi       itemp1,pulse_h_max
          brsh      norec
          cbr       flags,1<<header
          rjmp      int0_e

ir_recdata:
          cpi       itemp1,pulse_0_min
          brlo      norec
          cpi       itemp1,pulse_0_max
          brsh      ir_data1
          clc
          rjmp      ir_bitend

norec:    sec                            ; Set rising edge
          rcall     int0dir
          cbr       flags,1<<val_rec
int0_e:   out       sreg,inttemp
          reti                           ; exit

ir_data1: cpi       itemp1,pulse_1_min
          brlo      norec
          cpi       itemp1,pulse_1_max
          brsh      norec
          sec
ir_bitend:lds       itemp2,v_ircode
          rol       itemp2               ; !!!!
          inc       r_irbitpos
          ldi       itemp1,7
          sts       v_ircode,itemp2
          cp        r_irbitpos,itemp1    ; 1.st 8 bit$ only
          brne      int0_e
          sts       v_ircode,itemp2      ; real IR code
          ldi       temp1,ir_map_table
search_tab:rcall    w_eer                ; read eeprom
          cp        itemp2,temp2         ; search code
          breq      store
          cpi       temp1,0xFE           ; table end mark
          breq      norec
          inc       temp1
          inc       temp1
          inc       temp1                ; go to next row
          rjmp      search_tab

Store:    inc       temp1
          rcall     w_eer
          sts       v_irresult+1,temp2   ; Store remap code
          inc       temp1                ;
          out       eearh,null           ;
          out       eearl,temp1          ;
          rcall     w_eer                ;
          sts       v_irresult,temp2     ; Store param
          sec                            ; Set rising edge
          sts       val_rec,null         ;
          rcall     int0dir              ;
          sbr       flags,1<<ir_val_rec  ;
          ldi       temp1,5              ;
          rcall     wait                 ;
          rjmp      norec                ; !!! exit

eewrite:  out       eearh,null           ; EEPROM write
          out       eearl,temp1          ; temp1=addr
          out       eedr,temp2           ; temp2=stored byte
          sbi       eecr,eemwe
          sbi       eecr,eewe
w_eer:    sbic      eecr,eewe            ; EEPROM read
          rjmp      w_eer                ; temp1= addr
          out       eearh,null           ; temp2= ret.data
          out       eearl,temp1
          nop
          sbi       eecr,eere
          nop
          in        temp2,eedr
          ret
          ret

kbdru:    cbr       flags,1<<val_kbd     ;
          sts       real_kbd,null        ; clear keycode
          ldi       temp3,0xf0           ; initialise port A as i/o
          out       ddra, temp3          ; 4 out  4 in
          rcall     dpause               ;
          swap      temp3                ; key columns all low and
          nop                            ;
          out       porta, temp3         ; active pull ups on rows enabled
          rcall     dpause               ; allow time for port to settle
          in        temp1,pina           ;
          nop                            ;
          in        temp1,pina           ;
          nop                            ;
          nop                            ;
          ldi       temp4,0xff           ;
          sbrs      temp1,row1           ; find row of keypress
          ldi       temp4,0              ; and set row pointer
          sbrs      temp1,row2           ; row=3-0
          ldi       temp4,4              ; col=4-7
          sbrs      temp1,row3
          ldi       temp4,8
          sbrs      temp1,row4
          ldi       temp4,12
          ldi       temp3,0xff           ; no key pressed
          cpse      temp3,temp4          ; exit
          rjmp      next_scan
          rjmp      key_none

next_scan:ldi       temp3,0x0f           ; change port A i/o to
          out       ddra,temp3           ; find column press
          swap      temp3                ; enable pull ups and
          rcall     dpause
          out       porta,temp3          ; write 0s to rows
          rcall     dpause               ; allow time for port to settle
          in        temp1,pina
          nop
          in        temp1,pina
          nop
          nop
          nop
          sbrs      temp1,col1           ; find column of keypress
          ldi       temp3,0              ; and set col pointer
          sbrs      temp1,col2
          ldi       temp3,1
          sbrs      temp1,col3
          ldi       temp3,2
          sbrs      temp1,col4
          ldi       temp3,3
          add       temp4,temp3          ; merge row and col for pointer
          sts       v_keycode,temp4      ; save code & repeat scan
          ldi       temp3,0xf0           ; initialise port A as i/o
          out       ddra,temp3           ; 4 out  4 in
          swap      temp3                ; key columns all low and
          rcall     dpause
          out       porta,temp3          ; active pull ups on rows enabled
          rcall     dpause               ; allow time for port to settle
          in        temp1,pina
          nop
          in        temp1,pina
          nop
          nop
          nop
          sbrs      temp1,row1           ; find row of keypress
          ldi       temp4,0              ; and set row pointer
          sbrs      temp1,row2
          ldi       temp4,4
          sbrs      temp1,row3
          ldi       temp4,8
          sbrs      temp1,row4
          ldi       temp4,12
          ldi       temp3,0x0f           ; change port B i/o to
          out       ddra,temp3           ; find column press
          swap      temp3                ; enable pull ups and
          rcall     dpause
          out       porta,temp3          ; write 0s to rows
          rcall     dpause               ; allow time for port to settle
          in        temp1,pina
          nop
          in        temp1,pina
          nop
          nop
          nop
          sbrs      temp1,col1           ; find column of keypress
          ldi       temp3,0              ; and set col pointer
          sbrs      temp1,col2
          ldi       temp3,1
          sbrs      temp1,col3
          ldi       temp3,2
          sbrs      temp1,col4
          ldi       temp3,3
          add       temp4,temp3          ; merge row and col for pointer
          sts       v_keycode2,temp4
          lds       temp3,v_keycode
          cpse      temp3,temp4          ; code 1 = code 2 ?
          rjmp      key_none             ; NO = error
          sts       real_kbd,temp3       ; real kbdcode
          add       temp3,temp3          ;  temp3*3 !
          add       temp3,temp4          ; KEY code addr
          out       eearh,null
          out       EEARl,temp3          ; address EEPROM
          sbi       EECR,EERE            ; strobe EEPROM
          in        temp4,EEDR           ; read IR map
          sts       v_irresult,temp4     ; store IR header
          inc       temp3                ; next = keyb CODE (mapped)
          out       eearl,temp3
          out       eearh,null
          sbi       eecr,eere
          in        temp4,eedr
          sts       v_irresult+1,temp4   ; store keymap
          inc       temp3                ; next = PAR
          out       eearl,temp3
          sbi       eecr,eere
          in        temp4,eedr           ; read par.
          sts       V_IRRESULT+2,NULL    ; temp4
;         rcall     ir_test
          sbr       flags,1<<val_kbd
key_none: out       ddra,null
          out       porta,null
          ret

p03s:     ldi       temp1,12
wait:     clr       delayl
          clr       delayh               ; temp1*25msec
wait1:    dec       delayl               ; 24,64msec
          brne      wait1
          dec       delayh
          brne      wait1
          dec       temp1
          brne      wait1
          ret

reset:    nop
          cli
          clr       null                 ; clear RAM
          ldi       xaddrL,0x60
          ldi       xaddrH,0x60>>8
          ldi       yaddrL,(RAMend-0x5f)
          ldi       yaddrH,(RAMend-0x5f)>>8
clsRAM:   st        x+,null
          sbiw      yaddrl,1
          brne      clsRAM
          out       sph,null
          ldi       temp1,stacktop       ; 0x8F
          out       spl,temp1
          ldi       temp1,2
          rcall     wait
          ldi       temp1,1
          mov       eins,temp1
          ser       temp1
          mov       ff,temp1
          clr       lcdcmd               ; !
          clr       r_ircodecount
          out       ddrc,ff
          out       ddrb,ff              ; PB=OUT
          ldi       temp2,4
          out       tccr0,temp2          ; set timer 0  ck/256
          nop
          out       tcnt0,null           ; 0=256 ~ 8msec
          ldi       temp1,0x00000010b
          nop
          out       TIMSK,temp1
          ldi       temp1,2
          out       ddrd,temp1
          nop
          out       portd,temp2
          ldi       temp1,c_baud
          out       ubrr,temp1           ; set baudrate
          ldi       temp1,0x9c           ; dc
          out       ucr,temp1            ; 1001 1100
          out       usr,ff               ; ?
          ldi       temp1,br_stored
          rcall     w_eer                ; default val from EEPROM
          out       OCR1BH,null          ; PWM match regs.
          out       OCR1BL,temp2
          mov       stepreg,temp2
          ldi       temp1,0x21           ; Set 8 bit PWM na OC1B
          ldi       temp2,0b00000011     ; Set FCK/64 = 250 Hz
          out       TCCR1A,temp1
          nop
          out       TCCR1B,temp2         ; & PWM Run !
          nop
          ldi       temp1,0b00000011
          out       mcucr,temp1          ; int 0 rising edge
          ldi       temp1,0b01000000
          out       gimsk,temp1          ; int 0  enabled
          rcall     initdisplay
          ldi       temp1,2
          rcall     wait
          ldi       lcddata,0xc0
          rcall     writedisplay
          ldi       temp1,2
          rcall     wait
          clr       flags
          rcall     cleard
          ldi       temp1,2
          rcall     wait
          sei
          cbr       flags,1<<f_uart_rx
          ldi       temp3,8
clearsc:  push      temp3
          ldi       zaddrL,resetmel
          ldi       zaddrH,resetmel>>8
          rcall     sendtxt
          ldi       temp1,10
          rcall     wait
          pop       temp3
          dec       temp3
          brne      clearsc
extwsos:  cbr       flags,1<<f_uart_rx   ; wait for Rx data ..
wsos:     sbrs      flags,f_uart_rx
          rjmp      wsos
a0:       sbrc      flags,ir_val_rec     ; IRC valid
          rcall     ir_main              ;
          sbrc      flags,f_uart_rx      ; Rx complete !
          rcall     bmove                ; send rxbuffer
          sbrs      flags,f_uart_rx      ; wait until Rx + display complete
          rcall     kbdru                ; scan KBD
          sbrs      flags,val_kbd        ; key pressed ?
          rjmp      a0                   ;
          rcall     ir_main              ; send command
          ldi       temp1,10             ; 250 msec
          rcall     wait                 ;
a0a:      lds       temp1,real_kbd       ;
          cpi       temp1,0x0b           ; entermode?
          brne      a0                   ;
          sbr       flags,1<<f_uart_rx   ; MASTER !
sendn2:   sts       real_kbd,null        ;
          clr       temp1                ;
          push      temp1                ;
          rcall     tbset                ;
          pop       temp1
sendn3:   lds       temp2,real_kbd       ; normal KBD
          cpi       temp2,0x0b           ; repeat as entermode
          breq      texit                ; send command
          cpi       temp2,0x09           ; seek +
          breq      sendn1
          cpi       temp2,0x0a           ; step +
          breq      sendn1
          rjmp      extwsos              ; any other key exit !
sendn1:   sts       real_kbd,null
          inc       temp1                ; command counter
          cpi       temp1,7              ; 6 command$ max ..
          brsh      sendn2               ; skip to command 0 & wait
          push      temp1                ;
          rcall     sendn                ; next command ..
          pop       temp1
          rjmp      sendn3

texit:    tst       temp1                ; commands skiptable
          brne      sk10m
          ldi       temp1,0x1a           ; skip +10 track$
          rjmp      scs
sk10m:    cpi       temp1,1
          brne      b50p
          ldi       temp1,0x1b           ; skip -10 track$
          rjmp      scs
b50p:     cpi       temp1,2
          brne      b50m
          ldi       temp1,0x1e           ; skip +50 track$
          rjmp      scs
b50m:     cpi       temp1,3
          brne      timod
          ldi       temp1,0x1d           ; skip -50 track$
          rjmp      scs
timod:    cpi       temp1,4
          brne      set_bl
          ldi       temp1,0x0E           ; toggle timemode +/-
          rjmp      scs
set_bl:   cpi       temp1,5
          brne      setpar
          rjmp      bl_setco             ; Back light LCD setting
setpar:   cpi       temp1,6
          breq      scsa
          rjmp      extwsos
scsa:     ldi       temp1,0x0D		 ; store parms to EEPROM
scs:      sts       v_irresult+1,temp1   ; command send ready
          ldi       zaddrh,ok>>8
          ldi       zaddrl,ok
          rcall     sendtxt
          cbr       flags,1<<f_uart_rx   ; ! wait for next
scsw:     sbrs      flags,f_uart_rx      ; rx proc !
          rjmp      scsw
          ldi       temp1,4
          rcall     wait
          rcall     ir_main              ; send command + clear all
endcom:   ldi       zaddrh,ok>>8
          ldi       zaddrl,ok
          rcall     sendtxt
          rjmp      extwsos              ; wait next command ..

sendtxt:  add       zaddrL,zaddrL        ; adresa * 2 !!
          adc       zaddrH,zaddrH
          rcall     writepmtext          ; poslat hlasku
          rcall     display_main
          ret

bmove:    ldi       yaddrh,v_rxbuff>>8   ; move 32 bytes Rxbuff -> display
          ldi       yaddrl,v_rxbuff
          ldi       xaddrh,v_displaydata>>8
          ldi       xaddrl,v_displaydata
          ldi       temp1,32
rmove:    ld        temp2,y+
          st        x+,temp2
          dec       temp1
          brne      rmove
          rcall     display_main
          ret

tbset:    ldi       zaddrh,skip10p>>8 ; set textstring table
          ldi       zaddrl,skip10p
          sbiw      zaddrl,16
sendn:    adiw      zaddrl,16         ; table offset..
          push      zaddrh
          push      zaddrl
          rcall     sendtxt
          pop       zaddrl
          pop       zaddrh
wkb:      cbr       flags,1<<val_kbd  ; wait keypress
wkb3:     ldi       temp1,1
          rcall     wait
          rcall     kbdru
          sbrs      flags,val_kbd
          rjmp      wkb
          rcall     p03s
          ret

bl_setco: rcall     disp_pwm          ; nastaveni jasu displeje
bl_setc1: nop
wsetc:    rcall     wkb
          lds       temp1,real_kbd
          cpi       temp1,step_pl
          breq      pwm_pl
          cpi       temp1,skip_pl
          breq      pwm_pl
          cpi       temp1,step_mi
          breq      pwm_mi
          cpi       temp1,skip_mi
          breq      pwm_mi
          cpi       temp1,enter
          brne      wsetc
setc_end: ldi       zaddrh,dSTORE>>8
          ldi       zaddrl,dSTORE
          rcall     sendtxt
          rcall     wkb
          lds       temp1,real_kbd
          cpi       temp1,enter       ; store ?
          brne      de_end
          ldi       temp1,br_stored
          rcall     w_eer
          cp        temp2,stepreg
          breq      no_store
          mov       temp2,stepreg
          rcall     eewrite
no_store: mov       dd16uL,stepreg
          ldi       zaddrh,Stored>>8
          ldi       zaddrl,Stored
          add       zaddrL,zaddrL
          adc       zaddrH,zaddrH
          rcall     writepmtext
          ldi       zaddrh,v_displaydata>>8
          ldi       zaddrl,v_displaydata
          adiw      zaddrl,27             ; offset
          rcall     di_pwm_end            ; zobrazi NEW val ...
          ldi       temp1,60              ; cca 1,5 s
          rcall     wait
de_end:   rjmp      endcom                ; exit

pwm_pl:   ldi       temp2,diff            ; bright+
          add       temp2,stepreg
          mov       stepreg,temp2
          brcc      pwm_pl1
          subi      temp2,diff
          mov       stepreg,temp2
pwm_pl1:  rcall     set_pwm
          rcall     disp_pwm
          rjmp      bl_setc1

pwm_mi:   mov       temp2,stepreg         ; bright-
          ldi       temp1,diff
          sub       temp2,temp1
          mov       stepreg,temp2
          brcc      pwm_mi1
          add       temp2,temp1
          mov       stepreg,temp2
pwm_mi1:  rcall     set_pwm
          rcall     disp_pwm
          rjmp      bl_setc1

disp_pwm: ldi       zaddrh,bl_set>>8
          ldi       zaddrl,bl_set
          add       zaddrL,zaddrL
          adc       zaddrH,zaddrH
          rcall     writepmtext
          ldi       zaddrh,v_displaydata>>8
          ldi       zaddrl,v_displaydata
          adiw      zaddrl,23            ; offset
          mov       dd16uL,stepreg       ; prepocet na %
di_pwm_end:ldi      temp1,10             ; / 10
          mov       dv16ul,temp1
          clr       dv16uh
          clr       dd16uh
          rcall     div16u
          add       dres16ul,dres16ul
          add       dres16ul,dres16ul    ; * 4 !
          mov       dd16uL,dres16uL
          lsr       drem16uL
          add       dd16uL,drem16uL      ; + zbytek/2
          ldi       temp1,101
          cp        dd16uL,temp1
          brlo      wnum3ram
          dec       temp1
          mov       dd16uL,temp1         ; max 100%
wnum3ram: rcall     num3RAM              ; direct to RAM lcd buff
          ldi       lcddata,'%'          ; overwrite data ...
          st        z+,lcddata
          rcall     display_main
          ret

set_pwm:  out       OCR1BH,null          ; PWM match regs.
          out       OCR1BL,stepreg       ; pulse width 0-100%
          NOP
          ret

display_main:sbr    flags,1<<disp_up     ;
          rcall     disp_txt16
          cbr       flags,(1<<f_uart_rx)
          cbr       flags,1<<disp_up
          ret

disp_fb:  ldi       lcddata,0xc0
          rjmp      disp_fx
disp_fa:  ldi       lcddata,0x80         ; set DDRAM adr (0)
disp_fx:  clr       lcdcmd
          rcall     writedisplay
          ldi       lcdcmd,0xff
          ret

cleard:   ldi       lcddata,0x01         ; reset display
          rjmp      disp_fx

disp_txt16:rcall    disp_fa
          ldi       zaddrl,v_displaydata
          ldi       zaddrh,v_displaydata>>8
          clr       temp2
disp_txt16_1:ldi    temp1,16
disp_txt16_2:ld     lcddata,z+
          rcall     writedisplay
          dec       temp1
          brne      disp_txt16_2
          tst       temp2
          brne      disp_txt16_e
          rcall     disp_fb
          ser       temp2
          rjmp      disp_txt16_1
disp_txt16_e:
          ret

ir_main:  lds       temp2,v_irresult+1
          cpi       temp2,0x00           ; skip empty command
          breq      ir_main_e            ; (not mapped)
          lds       temp1,v_irresult
          sbi       ucr,txb8
          rcall     uart_addtx           ; 1 St byte =param. !!
          cbi       ucr,txb8
          lds       temp1,v_irresult+1   ; 2 Nd =code !
          rcall     uart_addtx
          lds       temp1,v_irresult+2
          rcall     uart_addtx
ir_main_e:cbr       flags,1<<ir_val_rec  ; reset command$
          rcall     p03s
          nop
          cbr       flags,1<<val_kbd     ;
;         rcall     ir_test
          sts       v_IRRESULT+1,null
          cbr       flags,1<<f_uart_rx
          ret

ir_test:  sbr       flags,(1<<f_uart_rx) ; blocked buffer
          rcall     cleard               ; disp_fa
          lds       dd16ul,v_ircode      ; IR_code (real)
          rcall     writenumber3
          ldi       lcddata,' '
          rcall     writedisplay
          lds       dd16ul,v_irresult+1  ; mapped command KB or IR
          rcall     writenumber3
          ldi       lcddata,' '
          rcall     writedisplay
          lds       dd16ul,real_kbd      ; real kbdcode
          rcall     writenumber3
          ldi       lcddata,' '
          rcall     writedisplay
          ret

enset:    ldi       temp2,10
          sbi       portb,lcd_b_en
ens_1:    dec       temp2
          brne      ens_1
          ret
enclear:  cbi       portb,lcd_b_en
          ldi       temp2,10
enc_1:    dec       temp2
          brne      enc_1
          ret

ensc:     rcall     enset
          rjmp      enclear

initdisplay:ldi     temp4,3              ; DB 4 & 5 =1
idi_1:    ldi       temp2,0x3            ; 00000011
          out       portb,temp2
          rcall     ensc
          ldi       temp1,1
          rcall     wait
          dec       temp4
          brne      idi_1                ; poslat 3* - OK
          ldi       temp2,0x2            ; 00000010
          out       portb,temp2
          rcall     ensc                 ; init end
          ldi       temp1,1
          rcall     wait
          ldi       lcddata,0x28         ; 00101000  ; Function Set Command
          rcall     writedisplay         ;
          ldi       lcddata,0x8          ; 00001000  ; Display off
          rcall     writedisplay
          ldi       lcddata,0x1          ; 00000001  ; Clear Display
          rcall     writedisplay
          ldi       lcddata,0xc          ; 00001100  ; Display ON
          rcall     writedisplay
          ret

dpause:   clr       delayl               ; 100 usec pauza
dpause1:  dec       delayl
          brne      dpause1
          ret

writenum: subi      lcddata,-'0'
writedisplay:push   temp2
          push      temp1
          mov       temp1,lcddata
          andi      temp1,0xf0
          swap      temp1                ; high nibble first
          nop
          out       portb,temp1
          rcall     dpause
          sbrc      lcdcmd,lcd_b_rs
          sbi       portb,lcd_b_rs
          rcall     dpause
          rcall     ensc                 ; set & clear ENABLE
          rcall     dpause
          mov       temp1,lcddata        ; write data/command & strobe
          andi      temp1,0x0f
          nop
          out       portb,temp1
          rcall     dpause
          sbrc      lcdcmd,lcd_b_rs
          sbi       portb,lcd_b_rs
          rcall     dpause
          rcall     ensc                 ; set & clear ENABLE
          clr       delayl               ; delay = 2 msec
          ldi       temp2,20
idi_3:    dec       delayl
          brne      idi_3
          dec       temp2
          brne      idi_3
          pop       temp1
          pop       temp2
          ret

writenumber3:ldi    temp1,100
          mov       dv16ul,temp1
          clr       dd16uh            ; max. 0xff
          clr       dv16uh
          rcall     div16u
          sbr       lcdcmd,(1<<lcd_b_rs)
          mov       lcddata,dres16ul
          mov       dd16ul,drem16ul
          clr       dd16uh
          ldi       temp1,10
          mov       dv16ul,temp1
          cpse      lcddata,null        ;Remove '0'
          rcall     writenum
          rcall     div16u
          mov       lcddata,dres16ul
          rcall     writenum
          mov       lcddata,drem16ul
          rcall     writenum
          ret

num3RAM:  ldi       temp1,100         ; adresa Z !
          mov       dv16ul,temp1
          clr       dv16uh
          clr       dd16uh            ; max. 0xff
          rcall     div16u
          mov       lcddata,dres16ul
          mov       dd16ul,drem16ul
          clr       dd16uh
          ldi       temp1,10
          mov       dv16ul,temp1
          subi      lcddata,-'0'
          push      lcddata            ; store val.
          cpi       lcddata,'0'        ; Remove '0'
          brne      nnum
          ldi       lcddata,' '
nnum:     st        z+,lcddata
          rcall     div16u
          pop       drem16uH           ; restore val.
          mov       lcddata,dres16ul
          subi      lcddata,-'0'
          cpi       lcddata,'0'
          brne      nnum1              ; if num=00
          cp        lcddata,drem16uH   ; Remove '0'
          brne      nnum1
          ldi       lcddata,' '
nnum1:    st        z+,lcddata
          mov       lcddata,drem16ul
          subi      lcddata,-'0'
          st        z+,lcddata
          ret

writepmtext:push    yaddrL
          push      yaddrH
          ldi       yaddrL,v_displaydata
          ldi       yaddrH,v_displaydata>>8
          lpm
          ldi       temp3,32
wrpm_1:   lpm
          adiw      zaddrl,1
          st        y+,pmreg
          dec       temp3
          brne      wrpm_1
          pop       yaddrH
          pop       yaddrL
          ret

div16u:   clr       drem16ul            ; clear remainder low byte
          sub       drem16uh,drem16uh   ; clear remainder high byte and carry
          ldi       dcnt16u,17          ; init loop counter
d16u_1:   rol       dd16ul              ; shift left dividend
          rol       dd16uh
          dec       dcnt16u             ; decrement counter
          brne      d16u_2              ; if done
          ret                           ; return
d16u_2:   rol       drem16ul            ; shift dividend into remainder
          rol       drem16uh
          sub       drem16ul,dv16ul     ; remainder = remainder - divisor
          sbc       drem16uh,dv16uh     ;
          brcc      d16u_3              ; if result negative
          add       drem16ul,dv16ul     ; restore remainder
          adc       drem16uh,dv16uh
          clc                           ; clear carry to be shifted into result
          rjmp      d16u_1              ; else
d16u_3:   sec                           ; set carry to be shifted into result
          rjmp      d16u_1

skiptables:
Skip10p:     .db     "Skip +10 Tracks                 "
Skip10m:     .db     "Skip -10 Tracks                 "
BIGSKIPp:    .db     "BIG Skip +50 Tr$                "
BIGSKIPm:    .db     "BIG Skip -50 Tr$                "
TIMEMODE:    .db     "Time Mode  +/-                  "
BL_SET:      .db     "Back light set..                "
REPEAT:      .db     "Vol.,bass,trebleStore ?         "
OK:          .db     "OK!                             "
RESETMEL:    .db     "MP3 Player V3.4 Wait Reset end.."
dSTORE:      .db     "Store as defaultvalue? Enter = Y"
Stored:      .db     "Data stored     New value =     "
;                                     N/L


; zde jsou kopie tabulek z hlavnho programu.Protoe tabulky
; zanaj od 0 (a 0 je przdn povel) je vyslan kd povelu
; vdy o 1 vt!
;            skok na podprogram      pijat povel
;IRCMDTAB:   rjmp  ICMD_PLAY         ;Code  1
;            rjmp  ICMD_STOP         ;Code  2
;            rjmp  ICMD_PAUSE        ;Code  3
;            rjmp  ICMD_VOL_UP       ;Code  4
;            rjmp  ICMD_VOL_DOWN     ;Code  5
;            rjmp  ICMD_TRACK_FWD    ;Code  6
;            rjmp  ICMD_TRACK_BWD    ;Code  7
;            rjmp  ICMD_NONE         ;Code  8
;            rjmp  ICMD_NONE         ;Code  9
;            rjmp  ICMD_NONE         ;Code  A
;            rjmp  ICMD_DISPLAY      ;Code  B      Time,File,Title,Artist,Album,Comment
;            rjmp  ICMD_SETUP        ;Code  C      Bass,treble,Balance
;            rjmp  ICMD_STORE        ;Code  D      Volume,Bass,Treble,Time+/-
;            rjmp  ICMD_TIME         ;Code  E      Toggle timemode +/-
;            rjmp  ICMD_EJECT        ;Code  F
;            rjmp  ICMD_NONE         ;Code  10
;            rjmp  ICMD_NONE         ;Code  11
;            rjmp  ICMD_NONE         ;Code  12
;            rjmp  ICMD_NONE         ;Code  13
;            rjmp  ICMD_NONE         ;Code  14
;            rjmp  ICMD_NONE         ;Code  15
;            rjmp  ICMD_NONE         ;Code  16
;            rjmp  ICMD_NONE         ;Code  17
;            rjmp  ICMD_NONE         ;Code  18
;            rjmp  ICMD_NONE         ;Code  19
;            rjmp  ICMD_SKIP10f      ;Code  1A
;            rjmp  ICMD_SKIP10b      ;Code  1B
;            rjmp  ICMD_NONE         ;Code  1C
;            rjmp  ICMD_SKIP50B      ;Code  1D
;            rjmp  ICMD_SKIP50F      ;Code  1E
;            rjmp  ICMD_REPEAT       ;Code  1F
;
;CD_DA_table:rjmp  DA_PLAY           ;Code  1
;            rjmp  DA_STOP           ;Code  2
;            rjmp  DA_PAUSE          ;Code  3
;            rjmp  DA_FORWARD        ;Code  4
;            rjmp  DA_BACKWARD       ;Code  5
;            rjmp  DA_FORWARD        ;Code  6
;            rjmp  DA_BACKWARD       ;Code  7
;            rjmp  DA_NONE           ;Code  8
;            rjmp  DA_NONE           ;Code  9
;            rjmp  DA_REPTRACK       ;Code  A
;            rjmp  DA_NONE           ;Code  B
;            rjmp  DA_NONE           ;Code  C
;            rjmp  DA_NONE           ;Code  D
;            rjmp  DA_NONE           ;Code  E
;            rjmp  DA_EJECT          ;Code  F
;            rjmp  DA_NONE           ;Code  10
;
;***** funkce podprogram viz komente u nich*********
;
     .eseg
     .org 0

; Pozn.k tabulkm:
; I kdy jsou tabulky 3 bytov,vyhodnocuji se jen prvn 2 byty
; Prostedn sloupec obou tabulek je kd povelu
; Klvesnice (4x4) se vyhodnocuje podle absolutnho poad
; tj klvesa s kdem 0 1.dek atd...
; IR povel ma na prvnm mst svj kd a na 2. kd povelu jako u klves.

KEYB_map_table:
;!!! kd 00 se NEPOSL !!!
;    irheader,mapcode,group        Keyb_code func.
     .db 0x11,0x01,0x01 ;          0x00 1    play/pause
     .db 0x11,0x03,0x01 ;          0x01 2
     .db 0x11,0x02,0x01 ;          0x02 3
     .db 0x11,14  ,0x01 ;          0x03 A
     .db 0x11,0x07,0x01 ;          0x04 4
     .db 0x11,12  ,0x01 ;          0x05 5    Prog
     .db 0x11,11  ,0x01 ;          0x06 6    Display
     .db 0x11,2   ,0x01 ;          0x07 B    Stop
     .db 0x11,0x05,0x01 ;          0x08 7
     .db 0x11,0x04,0x01 ;          0x09 8    Step+
     .db 0x11,06  ,0x01 ;          0x0a 9    Skip+
     .db 0x11,0   ,0x01 ;          0x0b C    Toggle(repeat)
     .db 0x11,15  ,0x01 ;          0x0c *    Eject
     .db 0x11,05  ,0x01 ;          0x0d 0    Step-
     .db 0x11,07  ,0x01 ;          0x0e +    Skip-
     .db 0x51,01  ,0x02 ;          0x0f D

; ! Poet pmo dostupnch funkc z klvesnice je dn potem klves,
; je teba vybrat ty dleitj !, zbytek pes Toggle nebo IR !

IR_MAP_TABLE:
;    IR code,Mapcode,par          IR_key     Func.
     .db   20 ,0x0A,00            ; Mute     Mute
     .db   0  ,0x11,00            ; 1        ICMD1
     .db   64 ,0x12,00            ; 2        ICMD2
     .db   32 ,0x13,00            ; 3        ICMD3
     .db   96 ,0x14,00            ; 4        ICMD4
     .db   16 ,0x15,00            ; 5        ICMD5
     .db   80 ,0x16,00            ; 6        ICMD6
     .db   48 ,0x17,00            ; 7        ICMD7
     .db   112,0x18,00            ; 8        ICMD8
     .db   8  ,0x19,00            ; 9        ICMD9
     .db   72 ,0x10,00            ; 0        ICMD0
     .db   24 ,0x00,00            ; 11       NONE!
     .db   88 ,0x00,00            ; 12       NONE!
     .db   31 ,0x00,00            ; memo     NONE!
     .db   4  ,0x06,00            ; prog+    TRACK_FWD
     .db   68 ,0x07,00            ; prog-    TRACK_BWD
     .db   10 ,0x0B,00            ; A        DISPLAY
     .db   74 ,0x0D,00            ; B        Store setting to EEPROM
     .db   116,0x0E,00            ; C        TIMEMODE+/-
     .db   105,0x00,00            ; D
     .db   25 ,0x0C,00            ; E        SETUP
     .db   89 ,0x00,00            ; F        NONE!
     .db   57 ,0x1E,00            ; G        SKIP+50
     .db   121,0x1F,00            ; H        SKIP-50
     .db   65 ,0x00,00            ; JAS+     NONE!
     .db   1  ,0x00,00            ; JAS-     NONE!
     .db   23 ,0x00,00            ; KONTR+   NONE!
     .db   87 ,0x00,00            ; KONTR-   NONE!
     .db   36 ,0x04,00            ; VOL+     VOL_UP
     .db   100,0x05,00            ; VOL-     VOL_DN
     .db   88 ,0x00,00            ; ST+
     .db   9  ,0x00,00            ; ST-
     .db   46 ,0x03,00            ; STOP     STOP
     .db   68 ,0x07,00            ; REWIN    TRACK_BWD
     .db   41 ,0x01,00            ; REC      PLAY/PAUSE
     .db   73 ,0x01,00            ; PLAY     PLAY/PAUSE
     .db   106,0x0F,00            ; EJECT    EJECT
     .db   4  ,0x06,00            ; WIND     TRACK_FWD
     .db   14 ,0x1A,00            ; 3*       SKIP+10
     .db   126,0x1B,00            ; PAUSE    SKIP-10
     .db   0xFE,0xFE,0xFE,0xFE    ; end mark

br_stored: .db    bri_default     ; default value for LCD brightness..
